from __future__ import print_function
import sys
import subprocess
import os
import re
import warnings
import csv


def get_distro_info(origin='Debian'):
    try:
        FileNotFoundException = FileNotFoundError
    except NameError:
        FileNotFoundException = IOError
    try:
        csvfile = open('/usr/share/distro-info/%s.csv' % origin.lower())
    except FileNotFoundException:
        csvfile = open('/usr/share/distro-info/debian.csv')
    reader = csv.DictReader(csvfile)
    global RELEASE_CODENAME_LOOKUP, RELEASES_ORDER, TESTING_CODENAME
    RELEASE_CODENAME_LOOKUP = {r['version']: r['series'] for r in reader if
        r['version']}
    RELEASES_ORDER = list(RELEASE_CODENAME_LOOKUP.items())
    RELEASES_ORDER.sort(key=lambda n: float(n[0]))
    RELEASES_ORDER = list(list(zip(*RELEASES_ORDER))[1])
    if origin.lower() == 'debian':
        TESTING_CODENAME = 'unknown.new.testing'
        RELEASES_ORDER.extend(['stable', 'proposed-updates', 'testing',
            'testing-proposed-updates', 'unstable', 'sid'])
    csvfile.close()


get_distro_info()


def lookup_codename(release, unknown=None):
    m = re.match('(\\d+)\\.(\\d+)(r(\\d+))?', release)
    if not m:
        return unknown
    if int(m.group(1)) < 7:
        shortrelease = '%s.%s' % m.group(1, 2)
    else:
        shortrelease = '%s' % m.group(1)
    return RELEASE_CODENAME_LOOKUP.get(shortrelease, unknown)


def valid_lsb_versions(version, module):
    if version == '3.0':
        return ['2.0', '3.0']
    elif version == '3.1':
        if module in ('desktop', 'qt4'):
            return ['3.1']
        elif module == 'cxx':
            return ['3.0', '3.1']
        else:
            return ['2.0', '3.0', '3.1']
    elif version == '3.2':
        if module == 'desktop':
            return ['3.1', '3.2']
        elif module == 'qt4':
            return ['3.1']
        elif module in ('printing', 'languages', 'multimedia'):
            return ['3.2']
        elif module == 'cxx':
            return ['3.0', '3.1', '3.2']
        else:
            return ['2.0', '3.0', '3.1', '3.2']
    elif version == '4.0':
        if module == 'desktop':
            return ['3.1', '3.2', '4.0']
        elif module == 'qt4':
            return ['3.1']
        elif module in ('printing', 'languages', 'multimedia'):
            return ['3.2', '4.0']
        elif module == 'security':
            return ['4.0']
        elif module == 'cxx':
            return ['3.0', '3.1', '3.2', '4.0']
        else:
            return ['2.0', '3.0', '3.1', '3.2', '4.0']
    elif version == '4.1':
        if module == 'desktop':
            return ['3.1', '3.2', '4.0', '4.1']
        elif module == 'qt4':
            return ['3.1']
        elif module in ('printing', 'languages', 'multimedia'):
            return ['3.2', '4.0', '4.1']
        elif module == 'security':
            return ['4.0', '4.1']
        elif module == 'cxx':
            return ['3.0', '3.1', '3.2', '4.0', '4.1']
        else:
            return ['2.0', '3.0', '3.1', '3.2', '4.0', '4.1']
    return [version]


try:
    set
except NameError:
    import sets
    set = sets.Set


def check_modules_installed():
    return []


longnames = {'v': 'version', 'o': 'origin', 'a': 'suite', 'c': 'component',
    'l': 'label'}


def parse_policy_line(data):
    retval = {}
    bits = data.split(',')
    for bit in bits:
        kv = bit.split('=', 1)
        if len(kv) > 1:
            k, v = kv[:2]
            if k in longnames:
                retval[longnames[k]] = v
    return retval


def release_index(x):
    suite = x[1].get('suite')
    if suite:
        if suite in RELEASES_ORDER:
            return int(len(RELEASES_ORDER) - RELEASES_ORDER.index(suite))
        else:
            try:
                return float(suite)
            except ValueError:
                return 0
    return 0


def compare_release(x, y):
    warnings.warn(
        'compare_release(x,y) is deprecated; please use the release_index(x) as key for sort() instead.'
        , DeprecationWarning, stacklevel=2)
    suite_x_i = release_index(x)
    suite_y_i = release_index(y)
    try:
        return suite_x_i - suite_y_i
    except TypeError:
        return (suite_x_i > suite_y_i) - (suite_x_i < suite_y_i)


def parse_apt_policy():
    data = []
    C_env = os.environ.copy()
    C_env['LC_ALL'] = 'C.UTF-8'
    policy = subprocess.Popen(['apt-cache', 'policy'], env=C_env, stdout=
        subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True).communicate()[
        0].decode('utf-8')
    for line in policy.split('\n'):
        line = line.strip()
        m = re.match('(-?\\d+)', line)
        if m:
            priority = int(m.group(1))
        if line.startswith('release'):
            bits = line.split(' ', 1)
            if len(bits) > 1:
                data.append((priority, parse_policy_line(bits[1])))
    return data


def guess_release_from_apt(origin='Debian', component='main', ignoresuites=
    'experimental', label='Debian', alternate_olabels={'Debian Ports': (
    'ftp.ports.debian.org', 'ftp.debian-ports.org')}):
    releases = parse_apt_policy()
    if not releases:
        return None
    releases = [x for x in releases if x[1].get('origin', '') == origin and
        x[1].get('suite', '') not in ignoresuites and x[1].get('component',
        '') == component and x[1].get('label', '') == label or x[1].get(
        'origin', '') in alternate_olabels and x[1].get('label', '') in
        alternate_olabels.get(x[1].get('origin', ''))]
    if not releases:
        return None
    releases.sort(key=lambda tuple: tuple[0], reverse=True)
    max_priority = releases[0][0]
    releases = [x for x in releases if x[0] == max_priority]
    releases.sort(key=release_index)
    return releases[0][1]


def guess_debian_release():
    distinfo = {}
    distinfo['ID'] = 'Debian'
    etc_dpkg_origins_default = os.environ.get('LSB_ETC_DPKG_ORIGINS_DEFAULT',
        '/etc/dpkg/origins/default')
    if os.path.exists(etc_dpkg_origins_default):
        try:
            with open(etc_dpkg_origins_default) as dpkg_origins_file:
                for line in dpkg_origins_file:
                    try:
                        header, content = line.split(': ', 1)
                        header = header.lower()
                        content = content.strip()
                        if header == 'vendor':
                            distinfo['ID'] = content
                    except ValueError:
                        pass
        except IOError as msg:
            print('Unable to open ' + etc_dpkg_origins_default + ':', str(
                msg), file=sys.stderr)
    get_distro_info(distinfo['ID'])
    kern = os.uname()[0]
    if kern in ('Linux', 'Hurd', 'NetBSD'):
        distinfo['OS'] = 'GNU/' + kern
    elif kern == 'FreeBSD':
        distinfo['OS'] = 'GNU/k' + kern
    elif kern in ('GNU/Linux', 'GNU/kFreeBSD'):
        distinfo['OS'] = kern
    else:
        distinfo['OS'] = 'GNU'
    distinfo['DESCRIPTION'] = '%(ID)s %(OS)s' % distinfo
    etc_debian_version = os.environ.get('LSB_ETC_DEBIAN_VERSION',
        '/etc/debian_version')
    if os.path.exists(etc_debian_version):
        try:
            with open(etc_debian_version) as debian_version:
                release = debian_version.read().strip()
        except IOError as msg:
            print('Unable to open ' + etc_debian_version + ':', str(msg),
                file=sys.stderr)
            release = 'unknown'
        if not release[0:1].isalpha():
            codename = lookup_codename(release, 'n/a')
            distinfo.update({'RELEASE': release, 'CODENAME': codename})
        elif release.endswith('/sid'):
            if release.rstrip('/sid').lower() != 'testing':
                global TESTING_CODENAME
                TESTING_CODENAME = release.rstrip('/sid')
            distinfo['RELEASE'] = 'testing/unstable'
        else:
            distinfo['RELEASE'] = release
    if not distinfo.get('CODENAME'):
        rinfo = guess_release_from_apt()
        if rinfo:
            release = rinfo.get('version')
            if release == '1.0' and rinfo.get('origin'
                ) == 'Debian Ports' and rinfo.get('label') in (
                'ftp.ports.debian.org', 'ftp.debian-ports.org'):
                release = None
                rinfo.update({'suite': 'unstable'})
            if release:
                codename = lookup_codename(release, 'n/a')
            else:
                release = rinfo.get('suite', 'unstable')
                if release == 'testing':
                    codename = TESTING_CODENAME
                else:
                    codename = 'sid'
            distinfo.update({'RELEASE': release, 'CODENAME': codename})
    if distinfo.get('RELEASE'):
        distinfo['DESCRIPTION'] += ' %(RELEASE)s' % distinfo
    if distinfo.get('CODENAME'):
        distinfo['DESCRIPTION'] += ' (%(CODENAME)s)' % distinfo
    return distinfo


def get_lsb_information():
    distinfo = {}
    etc_lsb_release = os.environ.get('LSB_ETC_LSB_RELEASE', '/etc/lsb-release')
    if os.path.exists(etc_lsb_release):
        try:
            with open(etc_lsb_release) as lsb_release_file:
                for line in lsb_release_file:
                    line = line.strip()
                    if not line:
                        continue
                    if not '=' in line:
                        continue
                    var, arg = line.split('=', 1)
                    if var.startswith('DISTRIB_'):
                        var = var[8:]
                        if arg.startswith('"') and arg.endswith('"'):
                            arg = arg[1:-1]
                        if arg:
                            distinfo[var] = arg.strip()
        except IOError as msg:
            print('Unable to open ' + etc_lsb_release + ':', str(msg), file
                =sys.stderr)
    return distinfo


def get_distro_information():
    lsbinfo = get_lsb_information()
    for key in ('ID', 'RELEASE', 'CODENAME', 'DESCRIPTION'):
        if key not in lsbinfo:
            distinfo = guess_debian_release()
            distinfo.update(lsbinfo)
            return distinfo
    else:
        return lsbinfo


def test():
    print(get_distro_information())
    print(check_modules_installed())


if __name__ == '__main__':
    test()
