from __future__ import absolute_import, unicode_literals
import io
import os
import sys
from collections import defaultdict
from functools import partial
from distutils.errors import DistutilsOptionError, DistutilsFileError
from setuptools.py26compat import import_module
from setuptools.extern.six import string_types


def read_configuration(filepath, find_others=False, ignore_option_errors=False
    ):
    u'Read given configuration file and returns options from it as a dict.\n\n    :param str|unicode filepath: Path to configuration file\n        to get options from.\n\n    :param bool find_others: Whether to search for other configuration files\n        which could be on in various places.\n\n    :param bool ignore_option_errors: Whether to silently ignore\n        options, values of which could not be resolved (e.g. due to exceptions\n        in directives such as file:, attr:, etc.).\n        If False exceptions are propagated as expected.\n\n    :rtype: dict\n    '
    from setuptools.dist import Distribution, _Distribution
    filepath = os.path.abspath(filepath)
    if not os.path.isfile(filepath):
        raise DistutilsFileError(u'Configuration file %s does not exist.' %
            filepath)
    current_directory = os.getcwd()
    os.chdir(os.path.dirname(filepath))
    try:
        dist = Distribution()
        filenames = dist.find_config_files() if find_others else []
        if filepath not in filenames:
            filenames.append(filepath)
        _Distribution.parse_config_files(dist, filenames=filenames)
        handlers = parse_configuration(dist, dist.command_options,
            ignore_option_errors=ignore_option_errors)
    finally:
        os.chdir(current_directory)
    return configuration_to_dict(handlers)


def configuration_to_dict(handlers):
    u'Returns configuration data gathered by given handlers as a dict.\n\n    :param list[ConfigHandler] handlers: Handlers list,\n        usually from parse_configuration()\n\n    :rtype: dict\n    '
    config_dict = defaultdict(dict)
    for handler in handlers:
        obj_alias = handler.section_prefix
        target_obj = handler.target_obj
        for option in handler.set_options:
            getter = getattr(target_obj, u'get_%s' % option, None)
            if getter is None:
                value = getattr(target_obj, option)
            else:
                value = getter()
            config_dict[obj_alias][option] = value
    return config_dict


def parse_configuration(distribution, command_options, ignore_option_errors
    =False):
    u'Performs additional parsing of configuration options\n    for a distribution.\n\n    Returns a list of used option handlers.\n\n    :param Distribution distribution:\n    :param dict command_options:\n    :param bool ignore_option_errors: Whether to silently ignore\n        options, values of which could not be resolved (e.g. due to exceptions\n        in directives such as file:, attr:, etc.).\n        If False exceptions are propagated as expected.\n    :rtype: list\n    '
    meta = ConfigMetadataHandler(distribution.metadata, command_options,
        ignore_option_errors)
    meta.parse()
    options = ConfigOptionsHandler(distribution, command_options,
        ignore_option_errors)
    options.parse()
    return [meta, options]


class ConfigHandler(object):
    u'Handles metadata supplied in configuration files.'
    section_prefix = None
    u'Prefix for config sections handled by this handler.\n    Must be provided by class heirs.\n\n    '
    aliases = {}
    u'Options aliases.\n    For compatibility with various packages. E.g.: d2to1 and pbr.\n    Note: `-` in keys is replaced with `_` by config parser.\n\n    '

    def __init__(self, target_obj, options, ignore_option_errors=False):
        sections = {}
        section_prefix = self.section_prefix
        for section_name, section_options in options.items():
            if not section_name.startswith(section_prefix):
                continue
            section_name = section_name.replace(section_prefix, u'').strip(u'.'
                )
            sections[section_name] = section_options
        self.ignore_option_errors = ignore_option_errors
        self.target_obj = target_obj
        self.sections = sections
        self.set_options = []

    @property
    def parsers(self):
        u'Metadata item name to parser function mapping.'
        raise NotImplementedError(u'%s must provide .parsers property' %
            self.__class__.__name__)

    def __setitem__(self, option_name, value):
        unknown = tuple()
        target_obj = self.target_obj
        option_name = self.aliases.get(option_name, option_name)
        current_value = getattr(target_obj, option_name, unknown)
        if current_value is unknown:
            raise KeyError(option_name)
        if current_value:
            return
        skip_option = False
        parser = self.parsers.get(option_name)
        if parser:
            try:
                value = parser(value)
            except Exception:
                skip_option = True
                if not self.ignore_option_errors:
                    raise
        if skip_option:
            return
        setter = getattr(target_obj, u'set_%s' % option_name, None)
        if setter is None:
            setattr(target_obj, option_name, value)
        else:
            setter(value)
        self.set_options.append(option_name)

    @classmethod
    def _parse_list(cls, value, separator=u','):
        u'Represents value as a list.\n\n        Value is split either by separator (defaults to comma) or by lines.\n\n        :param value:\n        :param separator: List items separator character.\n        :rtype: list\n        '
        if isinstance(value, list):
            return value
        if u'\n' in value:
            value = value.splitlines()
        else:
            value = value.split(separator)
        return [chunk.strip() for chunk in value if chunk.strip()]

    @classmethod
    def _parse_dict(cls, value):
        u'Represents value as a dict.\n\n        :param value:\n        :rtype: dict\n        '
        separator = u'='
        result = {}
        for line in cls._parse_list(value):
            key, sep, val = line.partition(separator)
            if sep != separator:
                raise DistutilsOptionError(
                    u'Unable to parse option value to dict: %s' % value)
            result[key.strip()] = val.strip()
        return result

    @classmethod
    def _parse_bool(cls, value):
        u'Represents value as boolean.\n\n        :param value:\n        :rtype: bool\n        '
        value = value.lower()
        return value in (u'1', u'true', u'yes')

    @classmethod
    def _parse_file(cls, value):
        u"Represents value as a string, allowing including text\n        from nearest files using `file:` directive.\n\n        Directive is sandboxed and won't reach anything outside\n        directory with setup.py.\n\n        Examples:\n            include: LICENSE\n            include: src/file.txt\n\n        :param str value:\n        :rtype: str\n        "
        if not isinstance(value, string_types):
            return value
        include_directive = u'file:'
        if not value.startswith(include_directive):
            return value
        current_directory = os.getcwd()
        filepath = value.replace(include_directive, u'').strip()
        filepath = os.path.abspath(filepath)
        if not filepath.startswith(current_directory):
            raise DistutilsOptionError(
                u'`file:` directive can not access %s' % filepath)
        if os.path.isfile(filepath):
            with io.open(filepath, encoding=u'utf-8') as f:
                value = f.read()
        return value

    @classmethod
    def _parse_attr(cls, value):
        u'Represents value as a module attribute.\n\n        Examples:\n            attr: package.attr\n            attr: package.module.attr\n\n        :param str value:\n        :rtype: str\n        '
        attr_directive = u'attr:'
        if not value.startswith(attr_directive):
            return value
        attrs_path = value.replace(attr_directive, u'').strip().split(u'.')
        attr_name = attrs_path.pop()
        module_name = u'.'.join(attrs_path)
        module_name = module_name or u'__init__'
        sys.path.insert(0, os.getcwd())
        try:
            module = import_module(module_name)
            value = getattr(module, attr_name)
        finally:
            sys.path = sys.path[1:]
        return value

    @classmethod
    def _get_parser_compound(cls, *parse_methods):
        u'Returns parser function to represents value as a list.\n\n        Parses a value applying given methods one after another.\n\n        :param parse_methods:\n        :rtype: callable\n        '

        def parse(value):
            parsed = value
            for method in parse_methods:
                parsed = method(parsed)
            return parsed
        return parse

    @classmethod
    def _parse_section_to_dict(cls, section_options, values_parser=None):
        u'Parses section options into a dictionary.\n\n        Optionally applies a given parser to values.\n\n        :param dict section_options:\n        :param callable values_parser:\n        :rtype: dict\n        '
        value = {}
        values_parser = values_parser or (lambda val: val)
        for key, (_, val) in section_options.items():
            value[key] = values_parser(val)
        return value

    def parse_section(self, section_options):
        u'Parses configuration file section.\n\n        :param dict section_options:\n        '
        for name, (_, value) in section_options.items():
            try:
                self[name] = value
            except KeyError:
                pass

    def parse(self):
        u'Parses configuration file items from one\n        or more related sections.\n\n        '
        for section_name, section_options in self.sections.items():
            method_postfix = u''
            if section_name:
                method_postfix = u'_%s' % section_name
            section_parser_method = getattr(self, (u'parse_section%s' %
                method_postfix).replace(u'.', u'__'), None)
            if section_parser_method is None:
                raise DistutilsOptionError(
                    u'Unsupported distribution option section: [%s.%s]' % (
                    self.section_prefix, section_name))
            section_parser_method(section_options)


class ConfigMetadataHandler(ConfigHandler):
    section_prefix = u'metadata'
    aliases = {u'home_page': u'url', u'summary': u'description',
        u'classifier': u'classifiers', u'platform': u'platforms'}
    strict_mode = False
    u'We need to keep it loose, to be partially compatible with\n    `pbr` and `d2to1` packages which also uses `metadata` section.\n\n    '

    @property
    def parsers(self):
        u'Metadata item name to parser function mapping.'
        parse_list = self._parse_list
        parse_file = self._parse_file
        return {u'platforms': parse_list, u'keywords': parse_list,
            u'provides': parse_list, u'requires': parse_list, u'obsoletes':
            parse_list, u'classifiers': self._get_parser_compound(
            parse_file, parse_list), u'license': parse_file, u'description':
            parse_file, u'long_description': parse_file, u'version': self.
            _parse_version}

    def parse_section_classifiers(self, section_options):
        u'Parses configuration file section.\n\n        :param dict section_options:\n        '
        classifiers = []
        for begin, (_, rest) in section_options.items():
            classifiers.append(u'%s :%s' % (begin.title(), rest))
        self[u'classifiers'] = classifiers

    def _parse_version(self, value):
        u'Parses `version` option value.\n\n        :param value:\n        :rtype: str\n\n        '
        version = self._parse_attr(value)
        if callable(version):
            version = version()
        if not isinstance(version, string_types):
            if hasattr(version, u'__iter__'):
                version = u'.'.join(map(str, version))
            else:
                version = u'%s' % version
        return version


class ConfigOptionsHandler(ConfigHandler):
    section_prefix = u'options'

    @property
    def parsers(self):
        u'Metadata item name to parser function mapping.'
        parse_list = self._parse_list
        parse_list_semicolon = partial(self._parse_list, separator=u';')
        parse_bool = self._parse_bool
        parse_dict = self._parse_dict
        return {u'zip_safe': parse_bool, u'use_2to3': parse_bool,
            u'include_package_data': parse_bool, u'package_dir': parse_dict,
            u'use_2to3_fixers': parse_list, u'use_2to3_exclude_fixers':
            parse_list, u'convert_2to3_doctests': parse_list, u'scripts':
            parse_list, u'eager_resources': parse_list, u'dependency_links':
            parse_list, u'namespace_packages': parse_list,
            u'install_requires': parse_list_semicolon, u'setup_requires':
            parse_list_semicolon, u'tests_require': parse_list_semicolon,
            u'packages': self._parse_packages, u'entry_points': self.
            _parse_file}

    def _parse_packages(self, value):
        u'Parses `packages` option value.\n\n        :param value:\n        :rtype: list\n        '
        find_directive = u'find:'
        if not value.startswith(find_directive):
            return self._parse_list(value)
        find_kwargs = self.parse_section_packages__find(self.sections.get(
            u'packages.find', {}))
        from setuptools import find_packages
        return find_packages(**find_kwargs)

    def parse_section_packages__find(self, section_options):
        u'Parses `packages.find` configuration file section.\n\n        To be used in conjunction with _parse_packages().\n\n        :param dict section_options:\n        '
        section_data = self._parse_section_to_dict(section_options, self.
            _parse_list)
        valid_keys = [u'where', u'include', u'exclude']
        find_kwargs = dict([(k, v) for k, v in section_data.items() if k in
            valid_keys and v])
        where = find_kwargs.get(u'where')
        if where is not None:
            find_kwargs[u'where'] = where[0]
        return find_kwargs

    def parse_section_entry_points(self, section_options):
        u'Parses `entry_points` configuration file section.\n\n        :param dict section_options:\n        '
        parsed = self._parse_section_to_dict(section_options, self._parse_list)
        self[u'entry_points'] = parsed

    def _parse_package_data(self, section_options):
        parsed = self._parse_section_to_dict(section_options, self._parse_list)
        root = parsed.get(u'*')
        if root:
            parsed[u''] = root
            del parsed[u'*']
        return parsed

    def parse_section_package_data(self, section_options):
        u'Parses `package_data` configuration file section.\n\n        :param dict section_options:\n        '
        self[u'package_data'] = self._parse_package_data(section_options)

    def parse_section_exclude_package_data(self, section_options):
        u'Parses `exclude_package_data` configuration file section.\n\n        :param dict section_options:\n        '
        self[u'exclude_package_data'] = self._parse_package_data(
            section_options)

    def parse_section_extras_require(self, section_options):
        u'Parses `extras_require` configuration file section.\n\n        :param dict section_options:\n        '
        parse_list = partial(self._parse_list, separator=u';')
        self[u'extras_require'] = self._parse_section_to_dict(section_options,
            parse_list)
